﻿using System.IO;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using C1.Silverlight;
using C1.Silverlight.FlexGrid;
using System.ComponentModel;
using System.Collections.Generic;
using System.IO.IsolatedStorage;
using System.Xml.Serialization;
using System.Windows.Data;

namespace FlexGrid_SaveState
{
    public partial class MainPage : UserControl
    {
        public MainPage()
        {
            InitializeComponent();

            // populate grid
            _flex.ItemsSource = Product.GetProducts(100);
            C1FlexGridFilter filter = new C1FlexGridFilter();
            C1FlexGridFilterService.SetFlexGridFilter(_flex, filter);

            LoadState();
            Application.Current.Exit += (s, e) =>
                {
                    SaveState(); 
                };        
        }

        private FlexGridSettings GetFlexGridSettings()
        {
            var layoutFileName = "ColumnLayout";
            IsolatedStorageFile isoFile = IsolatedStorageFile.GetUserStoreForApplication();
            if ( isoFile.FileExists(layoutFileName))
            {
                using (IsolatedStorageFileStream isoStream = new IsolatedStorageFileStream(layoutFileName, FileMode.Open, isoFile))
                {
                    try
                    {
                        XmlSerializer ser = CreateSerializer();
                        return (FlexGridSettings)ser.Deserialize(isoStream);
                    }
                    catch
                    {
                    }
                }
            }
            return new FlexGridSettings();
        }

        private static XmlSerializer CreateSerializer()
        {
            return new XmlSerializer(typeof(FlexGridSettings));
        }


        void SaveLayout_Click(object sender, RoutedEventArgs e)
        {
            SaveState(); 

        }
        void LoadLayout_Click(object sender, RoutedEventArgs e)
        {

            LoadState();  
        }

        public void SaveState()
        {
            var settings = new FlexGridSettings();

            settings.ColumnLayout = _flex.ColumnLayout;
            settings.FilterDefinition = C1FlexGridFilterService.GetFlexGridFilter(_flex).FilterDefinition;
            foreach (var sorts in _flex.CollectionView.SortDescriptions)
            {
                settings.SortsGroups.Sorts.Add(new SortedColumns() { ColumnName = sorts.PropertyName, Direction = sorts.Direction });

            }
            foreach (var groups in _flex.CollectionView.GroupDescriptions)
            {
                settings.SortsGroups.Groups.Add((groups as PropertyGroupDescription).PropertyName);
            }

            var layoutFileName = "ColumnLayout";
            IsolatedStorageFile isoFile = IsolatedStorageFile.GetUserStoreForApplication();
            using (IsolatedStorageFileStream isoStream = new IsolatedStorageFileStream(layoutFileName, FileMode.Create, isoFile))
            {
                XmlSerializer ser = CreateSerializer(); 
                ser.Serialize(isoStream, settings);
            }
        }

        public void LoadState()
        {
            var fgs = GetFlexGridSettings();
            if (fgs != null)
            {
                _flex.ColumnLayout = fgs.ColumnLayout;
                C1FlexGridFilterService.GetFlexGridFilter(_flex).FilterDefinition = fgs.FilterDefinition;
                foreach (var sorts in fgs.SortsGroups.Sorts)
                {
                    _flex.CollectionView.SortDescriptions.Add(new SortDescription(sorts.ColumnName, sorts.Direction));

                }
                foreach (var groups in fgs.SortsGroups.Groups)
                {
                    _flex.CollectionView.GroupDescriptions.Add(new PropertyGroupDescription(groups));
                }
            }
        }
    }

    public class SortedColumns
    {
        public string ColumnName { get; set; }
        public ListSortDirection Direction { get; set; }
    }

    public class ExtendedSettings
    {
        public  List<SortedColumns> Sorts{ get; set; }
  
        public List<string> Groups { get; set; }
      
    }

    public class FlexGridSettings
    {
        public FlexGridSettings()
        {
            this.SortsGroups = new ExtendedSettings();
            this.SortsGroups.Sorts = new List<SortedColumns>();
            this.SortsGroups.Groups = new List<string>();
        }

        public ExtendedSettings SortsGroups { get; set; }
        public string FilterDefinition { get; set; }
        public string ColumnLayout { get; set; }
    }
}
